import { Stack, Middleware } from './Stack';
import { Request, Response, RequestOptions, request } from './HTTPRequest';
import { ProgressData } from './utility';
export interface Context {
    request: Request;
    response: Response;
}
export interface ClientOptions extends RequestOptions {
    baseURI?: string;
    baseRequest?: typeof request;
}
export type MethodOptions = Omit<Request, 'method' | 'path' | 'headers' | 'body'>;
export interface DownloadOptions extends Pick<Request, 'headers' | 'withCredentials' | 'signal'> {
    chunkSize?: number;
    range?: [number?, number?];
}
export interface TransferProgress extends ProgressData {
    percent: number;
    buffer: ArrayBuffer;
}
export declare class HTTPClient<T extends Context> extends Stack<T> {
    baseURI: string;
    baseRequest: typeof request;
    options: RequestOptions;
    constructor({ baseURI, baseRequest, ...options }?: ClientOptions);
    defaultWare: Middleware<T>;
    use(...middlewares: Middleware<T>[]): this;
    request<B>(data: T['request']): Promise<Response<B>>;
    head(path: Request['path'], headers?: Request['headers'], options?: MethodOptions): Promise<Record<string, string | object>>;
    get<B>(path: Request['path'], headers?: Request['headers'], options?: MethodOptions): Promise<Response<B>>;
    post<B>(path: Request['path'], body?: Request['body'], headers?: Request['headers'], options?: MethodOptions): Promise<Response<B>>;
    put<B>(path: Request['path'], body?: Request['body'], headers?: Request['headers'], options?: MethodOptions): Promise<Response<B>>;
    patch<B>(path: Request['path'], body?: Request['body'], headers?: Request['headers'], options?: MethodOptions): Promise<Response<B>>;
    delete<B>(path: Request['path'], body?: Request['body'], headers?: Request['headers'], options?: MethodOptions): Promise<Response<B>>;
    download(path: Request['path'], { headers, chunkSize, range: [start, end], ...options }?: DownloadOptions): AsyncGenerator<TransferProgress>;
}
